# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import milvus_pb2 as milvus__pb2
from . import status_pb2 as status__pb2


class MilvusServiceStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
        """Constructor.

        Args:
          channel: A grpc.Channel.
        """
        self.CreateTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/CreateTable',
            request_serializer=milvus__pb2.TableSchema.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.HasTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/HasTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.BoolReply.FromString,
        )
        self.DescribeTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/DescribeTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.TableSchema.FromString,
        )
        self.CountTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/CountTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.TableRowCount.FromString,
        )
        self.ShowTables = channel.unary_unary(
            '/milvus.grpc.MilvusService/ShowTables',
            request_serializer=milvus__pb2.Command.SerializeToString,
            response_deserializer=milvus__pb2.TableNameList.FromString,
        )
        self.ShowTableInfo = channel.unary_unary(
            '/milvus.grpc.MilvusService/ShowTableInfo',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.TableInfo.FromString,
        )
        self.DropTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/DropTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.CreateIndex = channel.unary_unary(
            '/milvus.grpc.MilvusService/CreateIndex',
            request_serializer=milvus__pb2.IndexParam.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.DescribeIndex = channel.unary_unary(
            '/milvus.grpc.MilvusService/DescribeIndex',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.IndexParam.FromString,
        )
        self.DropIndex = channel.unary_unary(
            '/milvus.grpc.MilvusService/DropIndex',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.CreatePartition = channel.unary_unary(
            '/milvus.grpc.MilvusService/CreatePartition',
            request_serializer=milvus__pb2.PartitionParam.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.ShowPartitions = channel.unary_unary(
            '/milvus.grpc.MilvusService/ShowPartitions',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.PartitionList.FromString,
        )
        self.DropPartition = channel.unary_unary(
            '/milvus.grpc.MilvusService/DropPartition',
            request_serializer=milvus__pb2.PartitionParam.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.Insert = channel.unary_unary(
            '/milvus.grpc.MilvusService/Insert',
            request_serializer=milvus__pb2.InsertParam.SerializeToString,
            response_deserializer=milvus__pb2.VectorIds.FromString,
        )
        self.GetVectorByID = channel.unary_unary(
            '/milvus.grpc.MilvusService/GetVectorByID',
            request_serializer=milvus__pb2.VectorIdentity.SerializeToString,
            response_deserializer=milvus__pb2.VectorData.FromString,
        )
        self.GetVectorIDs = channel.unary_unary(
            '/milvus.grpc.MilvusService/GetVectorIDs',
            request_serializer=milvus__pb2.GetVectorIDsParam.SerializeToString,
            response_deserializer=milvus__pb2.VectorIds.FromString,
        )
        self.Search = channel.unary_unary(
            '/milvus.grpc.MilvusService/Search',
            request_serializer=milvus__pb2.SearchParam.SerializeToString,
            response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
        self.SearchByID = channel.unary_unary(
            '/milvus.grpc.MilvusService/SearchByID',
            request_serializer=milvus__pb2.SearchByIDParam.SerializeToString,
            response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
        self.SearchInFiles = channel.unary_unary(
            '/milvus.grpc.MilvusService/SearchInFiles',
            request_serializer=milvus__pb2.SearchInFilesParam.SerializeToString,
            response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
        self.Cmd = channel.unary_unary(
            '/milvus.grpc.MilvusService/Cmd',
            request_serializer=milvus__pb2.Command.SerializeToString,
            response_deserializer=milvus__pb2.StringReply.FromString,
        )
        self.DeleteByID = channel.unary_unary(
            '/milvus.grpc.MilvusService/DeleteByID',
            request_serializer=milvus__pb2.DeleteByIDParam.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.PreloadTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/PreloadTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.Flush = channel.unary_unary(
            '/milvus.grpc.MilvusService/Flush',
            request_serializer=milvus__pb2.FlushParam.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.Compact = channel.unary_unary(
            '/milvus.grpc.MilvusService/Compact',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )


class MilvusServiceServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def CreateTable(self, request, context):
        """*
        @brief This method is used to create table

        @param TableSchema, use to provide table information to be created.

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HasTable(self, request, context):
        """*
        @brief This method is used to test table existence.

        @param TableName, table name is going to be tested.

        @return BoolReply
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeTable(self, request, context):
        """*
        @brief This method is used to get table schema.

        @param TableName, target table name.

        @return TableSchema
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CountTable(self, request, context):
        """*
        @brief This method is used to get table schema.

        @param TableName, target table name.

        @return TableRowCount
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ShowTables(self, request, context):
        """*
        @brief This method is used to list all tables.

        @param Command, dummy parameter.

        @return TableNameList
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ShowTableInfo(self, request, context):
        """*
        @brief This method is used to get table detail information.

        @param TableName, target table name.

        @return TableInfo
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropTable(self, request, context):
        """*
        @brief This method is used to delete table.

        @param TableName, table name is going to be deleted.

        @return TableNameList
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateIndex(self, request, context):
        """*
        @brief This method is used to build index by table in sync mode.

        @param IndexParam, index paramters.

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeIndex(self, request, context):
        """*
        @brief This method is used to describe index

        @param TableName, target table name.

        @return IndexParam
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropIndex(self, request, context):
        """*
        @brief This method is used to drop index

        @param TableName, target table name.

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreatePartition(self, request, context):
        """*
        @brief This method is used to create partition

        @param PartitionParam, partition parameters.

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ShowPartitions(self, request, context):
        """*
        @brief This method is used to show partition information

        @param TableName, target table name.

        @return PartitionList
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropPartition(self, request, context):
        """*
        @brief This method is used to drop partition

        @param PartitionParam, target partition.

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Insert(self, request, context):
        """*
        @brief This method is used to add vector array to table.

        @param InsertParam, insert parameters.

        @return VectorIds
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetVectorByID(self, request, context):
        """*
        @brief This method is used to get vector data by id.

        @param VectorIdentity, target vector id.

        @return VectorData
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetVectorIDs(self, request, context):
        """*
        @brief This method is used to get vector ids from a segment

        @param GetVectorIDsParam, target table and segment

        @return VectorIds
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Search(self, request, context):
        """*
        @brief This method is used to query vector in table.

        @param SearchParam, search parameters.

        @return TopKQueryResult
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SearchByID(self, request, context):
        """*
        @brief This method is used to query vector by id.

        @param SearchByIDParam, search parameters.

        @return TopKQueryResult
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SearchInFiles(self, request, context):
        """*
        @brief This method is used to query vector in specified files.

        @param SearchInFilesParam, search in files paremeters.

        @return TopKQueryResult
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Cmd(self, request, context):
        """*
        @brief This method is used to give the server status.

        @param Command, command string

        @return StringReply
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteByID(self, request, context):
        """*
        @brief This method is used to delete vector by id

        @param DeleteByIDParam, delete parameters.

        @return status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PreloadTable(self, request, context):
        """*
        @brief This method is used to preload table

        @param TableName, target table name.

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Flush(self, request, context):
        """*
        @brief This method is used to flush buffer into storage.

        @param FlushParam, flush parameters

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Compact(self, request, context):
        """*
        @brief This method is used to compact table

        @param TableName, target table name.

        @return Status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MilvusServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'CreateTable': grpc.unary_unary_rpc_method_handler(
            servicer.CreateTable,
            request_deserializer=milvus__pb2.TableSchema.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'HasTable': grpc.unary_unary_rpc_method_handler(
            servicer.HasTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.BoolReply.SerializeToString,
        ),
        'DescribeTable': grpc.unary_unary_rpc_method_handler(
            servicer.DescribeTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.TableSchema.SerializeToString,
        ),
        'CountTable': grpc.unary_unary_rpc_method_handler(
            servicer.CountTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.TableRowCount.SerializeToString,
        ),
        'ShowTables': grpc.unary_unary_rpc_method_handler(
            servicer.ShowTables,
            request_deserializer=milvus__pb2.Command.FromString,
            response_serializer=milvus__pb2.TableNameList.SerializeToString,
        ),
        'ShowTableInfo': grpc.unary_unary_rpc_method_handler(
            servicer.ShowTableInfo,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.TableInfo.SerializeToString,
        ),
        'DropTable': grpc.unary_unary_rpc_method_handler(
            servicer.DropTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'CreateIndex': grpc.unary_unary_rpc_method_handler(
            servicer.CreateIndex,
            request_deserializer=milvus__pb2.IndexParam.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'DescribeIndex': grpc.unary_unary_rpc_method_handler(
            servicer.DescribeIndex,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.IndexParam.SerializeToString,
        ),
        'DropIndex': grpc.unary_unary_rpc_method_handler(
            servicer.DropIndex,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'CreatePartition': grpc.unary_unary_rpc_method_handler(
            servicer.CreatePartition,
            request_deserializer=milvus__pb2.PartitionParam.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'ShowPartitions': grpc.unary_unary_rpc_method_handler(
            servicer.ShowPartitions,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.PartitionList.SerializeToString,
        ),
        'DropPartition': grpc.unary_unary_rpc_method_handler(
            servicer.DropPartition,
            request_deserializer=milvus__pb2.PartitionParam.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'Insert': grpc.unary_unary_rpc_method_handler(
            servicer.Insert,
            request_deserializer=milvus__pb2.InsertParam.FromString,
            response_serializer=milvus__pb2.VectorIds.SerializeToString,
        ),
        'GetVectorByID': grpc.unary_unary_rpc_method_handler(
            servicer.GetVectorByID,
            request_deserializer=milvus__pb2.VectorIdentity.FromString,
            response_serializer=milvus__pb2.VectorData.SerializeToString,
        ),
        'GetVectorIDs': grpc.unary_unary_rpc_method_handler(
            servicer.GetVectorIDs,
            request_deserializer=milvus__pb2.GetVectorIDsParam.FromString,
            response_serializer=milvus__pb2.VectorIds.SerializeToString,
        ),
        'Search': grpc.unary_unary_rpc_method_handler(
            servicer.Search,
            request_deserializer=milvus__pb2.SearchParam.FromString,
            response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
        ),
        'SearchByID': grpc.unary_unary_rpc_method_handler(
            servicer.SearchByID,
            request_deserializer=milvus__pb2.SearchByIDParam.FromString,
            response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
        ),
        'SearchInFiles': grpc.unary_unary_rpc_method_handler(
            servicer.SearchInFiles,
            request_deserializer=milvus__pb2.SearchInFilesParam.FromString,
            response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
        ),
        'Cmd': grpc.unary_unary_rpc_method_handler(
            servicer.Cmd,
            request_deserializer=milvus__pb2.Command.FromString,
            response_serializer=milvus__pb2.StringReply.SerializeToString,
        ),
        'DeleteByID': grpc.unary_unary_rpc_method_handler(
            servicer.DeleteByID,
            request_deserializer=milvus__pb2.DeleteByIDParam.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'PreloadTable': grpc.unary_unary_rpc_method_handler(
            servicer.PreloadTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'Flush': grpc.unary_unary_rpc_method_handler(
            servicer.Flush,
            request_deserializer=milvus__pb2.FlushParam.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'Compact': grpc.unary_unary_rpc_method_handler(
            servicer.Compact,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'milvus.grpc.MilvusService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))



import copy
import functools

from . import __version__
from .types import IndexType, MetricType
from .check import check_pass_param
from .grpc_handler import GrpcHandler
from .http_handler import HttpHandler
from .exceptions import ParamError, NotConnectError


def check_connect(f):
    @functools.wraps(f)
    def wrapper(self, *args, **kwargs):
        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        return f(self, *args, **kwargs)

    return wrapper


class Milvus:
    def __init__(self, host=None, port=None, handler="GRPC", **kwargs):
        if handler == "GRPC":
            self._handler = GrpcHandler(host=host, port=port, **kwargs)
        elif handler == "HTTP":
            self._handler = HttpHandler(host=host, port=port, **kwargs)
        else:
            raise ParamError("Unknown handler options, please use \'GRPC\' or \'HTTP\'")

    def __enter__(self):
        self._handler.__enter__()

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._handler.__exit__(exc_type, exc_val, exc_tb)

    def set_hook(self, **kwargs):
        return self._handler.set_hook(**kwargs)

    @property
    def status(self):
        return self._handler.status

    @property
    def handler(self):
        if isinstance(self._handler, GrpcHandler):
            return "GRPC"

        if isinstance(self._handler, HttpHandler):
            return "HTTP"

        return "NULL"

    def connect(self, host=None, port=None, uri=None, timeout=1):
        """
        Connects to the Milvus server.

        :type  host: str
        :type  port: str
        :type  uri: str
        :type  timeout: float
        :param host: (Optional) Ip address of the Milvus server. The default is `127.0.0.1`.
        :param port: (Optional) Port of the Milvus server. The default is `19530`.
        :param uri: (Optional) URI of the Milvus server. Only TCP is supported. The default is `tcp://127.0.0.1:19530`.

        :param timeout: (Optional) Connection timeout in milliseconds. The default is 3000.

        :return: Indicates whether the connection is successful.
        :rtype: Status
        
        :raises: NotConnectError
        """
        return self._handler.connect(host, port, uri, timeout)

    def connected(self):
        """
        Checks whether the client is connected to the Milvus server.

        :return: Whether the client is connected to the Milvus server.
        :rtype: bool
        """
        return self._handler.connected()

    @check_connect
    def disconnect(self):
        """
        Disconnects from the Milvus server.

        :return: Whether the client is disconnected from the Milvus server.
        :rtype: Status
        """
        return self._handler.disconnect()

    def client_version(self):
        """
        Returns the version of the client.

        :return: Version of the client.

        :rtype: (str)
        """
        return __version__

    @check_connect
    def server_status(self, timeout=10):
        """
        Returns the status of the Milvus server.

        :return:
            Status: Whether the operation is successful.

            str : Status of the Milvus server.

        :rtype: (Status, str)
        """
        return self._cmd("status", timeout)

    @check_connect
    def server_version(self, timeout=10):
        """
       Returns the version of the Milvus server.

       :return:
           Status: Whether the operation is successful.

           str : Version of the Milvus server.

       :rtype: (Status, str)
       """

        return self._cmd("version", timeout)

    @check_connect
    def _cmd(self, cmd, timeout=10):
        check_pass_param(cmd=cmd)

        return self._handler._cmd(cmd, timeout)

    @check_connect
    def create_collection(self, param, timeout=10):
        """
        Creates a collection.

        :type  param: dict
        :param param: Information needed to create a collection.

                `param={'collection_name': 'name',
                                'dimension': 16,
                                'index_file_size': 1024 (default)，
                                'metric_type': Metric_type.L2 (default)
                                }`

        :param timeout: Timeout in seconds.
        :type  timeout: double

        :return: Whether the operation is successful.
        :rtype: Status
        """
        if not isinstance(param, dict):
            raise ParamError('Param type incorrect, expect {} but get {} instead'
                             .format(type(dict), type(param)))

        collection_param = copy.deepcopy(param)

        if 'collection_name' not in collection_param:
            raise ParamError('collection_name is required')
        collection_name = collection_param["collection_name"]
        collection_param.pop('collection_name')

        if 'dimension' not in collection_param:
            raise ParamError('dimension is required')
        dim = collection_param["dimension"]
        collection_param.pop("dimension")

        index_file_size = collection_param.get('index_file_size', 1024)
        collection_param.pop('index_file_size', None)

        metric_type = collection_param.get('metric_type', MetricType.L2)
        collection_param.pop('metric_type', None)

        check_pass_param(collection_name=collection_name, dimension=dim, index_file_size=index_file_size,
                         metric_type=metric_type)

        return self._handler.create_table(collection_name, dim, index_file_size, metric_type, collection_param, timeout)

    @check_connect
    def has_collection(self, collection_name, timeout=10):
        """

        Checks whether a collection exists.

        :param collection_name: Name of the collection to check.
        :type  collection_name: str
        :param timeout: Timeout in seconds.
        :type  timeout: int

        :return:
            Status: indicate whether the operation is successful.
            bool if given collection_name exists

        """
        check_pass_param(collection_name=collection_name)
        return self._handler.has_table(collection_name, timeout)

    @check_connect
    def describe_collection(self, collection_name, timeout=10):
        """
        Returns information of a collection.

        :type  collection_name: str
        :param collection_name: Name of the collection to describe.

        :returns: (Status, table_schema)
            Status: indicate if query is successful
            table_schema: return when operation is successful
        :rtype: (Status, TableSchema)
        """
        check_pass_param(collection_name=collection_name)
        return self._handler.describe_table(collection_name, timeout)

    @check_connect
    def count_collection(self, collection_name, timeout=10):
        """
        Returns the number of vectors in a collection.

        :type  collection_name: str
        :param collection_name: target table name.

        :returns:
            Status: indicate if operation is successful

            res: int, table row count
        """
        check_pass_param(collection_name=collection_name)
        return self._handler.count_table(collection_name, timeout)

    @check_connect
    def show_collections(self, timeout=10):
        """
        Returns information of all collections.

        :return:
            Status: indicate if this operation is successful

            collections: list of table names, return when operation
                    is successful
        :rtype:
            (Status, list[str])
        """
        return self._handler.show_tables(timeout)

    @check_connect
    def collection_info(self, collection_name, timeout=10):
        return self._handler.show_table_info(collection_name, timeout)

    @check_connect
    def preload_collection(self, collection_name, timeout=None):
        """
        Loads a collection for caching.

        :type collection_name: str
        :param collection_name: table to preload

        :returns:
            Status:  indicate if invoke is successful
        """
        check_pass_param(collection_name=collection_name)
        return self._handler.preload_table(collection_name, timeout)

    @check_connect
    def drop_collection(self, collection_name, timeout=10):
        """
        Deletes a collection by name.

        :type  collection_name: str
        :param collection_name: Name of the table being deleted

        :return: Status, indicate if operation is successful
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name)
        return self._handler.drop_table(collection_name, timeout)

    @check_connect
    def insert(self, collection_name, records, ids=None, partition_tag=None, params=None, timeout=-1):
        """
        Insert vectors to a collection.

        :param ids: list of id
        :type  ids: list[int]

        :type  collection_name: str
        :param collection_name: collection name been inserted

        :type  records: list[list[float]]

                `example records: [[1.2345],[1.2345]]`

                `OR using Prepare.records`

        :param records: list of vectors been inserted

        :type partition_tag: str or None.
            If partition_tag is None, vectors will be inserted to the collection rather than partitions.

        :param partition_tag: the tag string of a collection

        :type

        :type  timeout: int
        :param timeout: time waiting for server response

        :returns:
            Status: indicate whether vectors are inserted successfully.
            ids: IDs of the inserted vectors.
        :rtype: (Status, list(int))
        """
        check_pass_param(collection_name=collection_name, records=records,
                         ids=ids, partition_tag=partition_tag)

        if ids is not None and len(records) != len(ids):
            raise ParamError("length of vectors do not match that of ids")

        params = params or dict()
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")

        return self._handler.insert(collection_name, records, ids, partition_tag, params, timeout)

    @check_connect
    def get_vector_by_id(self, collection_name, vector_id, timeout=None):
        check_pass_param(collection_name=collection_name, ids=[vector_id])

        return self._handler.get_vector_by_id(collection_name, vector_id, timeout=timeout)

    @check_connect
    def get_vector_ids(self, collection_name, segment_name, timeout=None):
        check_pass_param(collection_name=collection_name)
        check_pass_param(collection_name=segment_name)

        return self._handler.get_vector_ids(collection_name, segment_name, timeout)

    @check_connect
    def create_index(self, collection_name, index_type=None, params=None, timeout=None):
        """
        Creates index for a collection.

        :param collection_name: Collection used to create index.
        :type collection_name: str
        :param index: index params
        :type index: dict

            index_param can be None

            `example (default) param={'index_type': IndexType.FLAT,
                            'nlist': 16384}`

        :param timeout: grpc request timeout.

            if `timeout` = -1, method invoke a synchronous call, waiting util grpc response
            else method invoke a asynchronous call, timeout work here

        :type  timeout: int

        :return: Whether the operation is successful.
        """
        _index_type = IndexType.FLAT if index_type is None else index_type
        check_pass_param(collection_name=collection_name, index_type=_index_type)

        params = params or dict()
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")

        return self._handler.create_index(collection_name, _index_type, params, timeout)

    @check_connect
    def describe_index(self, collection_name, timeout=10):
        """
        Show index information of a collection.

        :type collection_name: str
        :param collection_name: table name been queried

        :returns:
            Status:  Whether the operation is successful.
            IndexSchema:

        """
        check_pass_param(collection_name=collection_name)
        return self._handler.describe_index(collection_name, timeout)

    @check_connect
    def drop_index(self, collection_name, timeout=10):
        """
        Removes an index.

        :param collection_name: target collection name.
        :type collection_name: str

        :return:
            Status: Whether the operation is successful.

        ：:rtype: Status
        """
        check_pass_param(collection_name=collection_name)
        return self._handler.drop_index(collection_name, timeout)

    @check_connect
    def create_partition(self, collection_name, partition_tag, timeout=10):
        """
        create a partition for a collection. 

        :param collection_name: Name of the collection.
        :type  collection_name: str

        :param partition_name: Name of the partition.
        :type  partition_name: str

        :param partition_tag: Name of the partition tag.
        :type  partition_tag: str

        :param timeout: time waiting for response.
        :type  timeout: int

       :return:
            Status: Whether the operation is successful.

        """
        check_pass_param(collection_name=collection_name, partition_tag=partition_tag)

        return self._handler.create_partition(collection_name, partition_tag, timeout)

    @check_connect
    def show_partitions(self, collection_name, timeout=10):
        """
        Show all partitions in a collection.

        :param collection_name: target table name.
        :type  collection_name: str

        :param timeout: time waiting for response.
        :type  timeout: int

        :return:
            Status: Whether the operation is successful.
            partition_list:

        """
        check_pass_param(collection_name=collection_name)
        return self._handler.show_partitions(collection_name, timeout)

    @check_connect
    def drop_partition(self, collection_name, partition_tag, timeout=10):
        """
        Deletes a partition in a collection.

        :param collection_name: Collection name.
        :type  collection_name: str

        :param partition_tag: Partition name.
        :type  partition_tag: str

        :param timeout: time waiting for response.
        :type  timeout: int

        :return:
            Status: Whether the operation is successful.

        """
        check_pass_param(collection_name=collection_name, partition_tag=partition_tag)
        return self._handler.drop_partition(collection_name, partition_tag, timeout)

    @check_connect
    def search(self, collection_name, top_k, query_records, partition_tags=None, params=None):
        """
        Search vectors in a collection.

        :param collection_name: Name of the collection.
        :type  collection_name: str

        :param top_k: number of vertors which is most similar with query vectors
        :type  top_k: int

        :param nprobe: cell number of probe
        :type  nprobe: int

        :param query_records: vectors to query
        :type  query_records: list[list[float32]]

        :param partition_tags: tags to search
        :type  partition_tags: list

        :return
            Status: Whether the operation is successful.
            result: query result

        :rtype: (Status, TopKQueryResult)

        """
        check_pass_param(collection_name=collection_name, topk=top_k,
                         records=query_records, partition_tag_array=partition_tags)

        params = params or dict()
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")

        return self._handler.search(collection_name, top_k, query_records, partition_tags, params)

    @check_connect
    def search_in_files(self, collection_name, file_ids, query_records, top_k, params=None):
        """
        Searches for vectors in specific files of a collection.

        The Milvus server stores vector data into multiple files. Searching for vectors in specific files is a
        method used in Mishards. Obtain more detail about Mishards, see
        <a href="https://github.com/milvus-io/milvus/tree/master/shards">

        :type  collection_name: str
        :param collection_name: table name been queried

        :type  file_ids: list[str] or list[int]
        :param file_ids: Specified files id array

        :type  query_records: list[list[float]]
        :param query_records: all vectors going to be queried

        :param query_ranges: Optional ranges for conditional search.
            If not specified, search in the whole table

        :type  top_k: int
        :param top_k: how many similar vectors will be searched

        :returns:
            Status:  indicate if query is successful
            results: query result

        :rtype: (Status, TopKQueryResult)
        """
        check_pass_param(collection_name=collection_name, topk=top_k, records=query_records)

        params = params or dict()
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")

        return self._handler.search_in_files(collection_name, file_ids,
                                             query_records, top_k, params)

    @check_connect
    def delete_by_id(self, collection_name, id_array, timeout=None):
        """
        Deletes vectors in a collection by vector ID.

        """
        check_pass_param(collection_name=collection_name, ids=id_array)

        return self._handler.delete_by_id(collection_name, id_array, timeout)

    @check_connect
    def flush(self, collection_name_array=None):
        """
        Flushes vector data in one collection or multiple collections to disk.

        """
        if collection_name_array is None:
            return self._handler.flush([])

        if not isinstance(collection_name_array, list):
            raise ParamError("Collection name array must be type of list")

        if len(collection_name_array) <= 0:
            raise ParamError("Collection name array is not allowed to be empty")

        for name in collection_name_array:
            check_pass_param(collection_name=name)

        return self._handler.flush(collection_name_array)

    @check_connect
    def compact(self, collection_name, timeout=None):
        """
        Compacts segments in a collection.

        """
        check_pass_param(collection_name=collection_name)

        return self._handler.compact(collection_name, timeout)

    @check_connect
    def get_config(self, parent_key, child_key):
        """
        Gets Milvus configurations.

        """
        cmd = "get_config {}.{}".format(parent_key, child_key)

        return self._cmd(cmd)

    @check_connect
    def set_config(self, parent_key, child_key, value):
        """
        Sets Milvus configurations.

        """
        cmd = "set_config {}.{} {}".format(parent_key, child_key, value)

        return self._cmd(cmd)

    # In old version of pymilvus, some methods are different from the new.
    # apply alternative method name for compatibility

    # get_collection_row_count = count_collection
    # delete_collection = drop_collection
    add_vectors = insert
    search_vectors = search
    search_vectors_in_files = search_in_files
